%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [ a,b,c,d ] = coeff_ARQ_ff_global( z,Lmax,i,j,CC1,CC2,CC3,R3)
% This function calculates the coefficients to solve 
% for optimal power allocation using geometric programming
% for ARQ FAST FADING channel
%
% Author: Mohit K. Sharma
% Email: mohit.sharma987@gmail.com
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [ a,b,c,d ] = coeff_ARQ_ff_global( z,Lmax,i,j,CC1,CC2,CC3,R3)
%This function calculates the coefficients to solve for optimal power allocation using GP 
%for ARQ FAST FADING channel
z11=z(1); z21=z(5);
z12=z(2); z22=z(6);
z13=z(3); z23=z(7);
z14=z(4); z24=z(8);

if ((i==1)&&(j==1))
    DEN1 = 1+((z14^(-1))*(z24^(-1)));
    DEN2 = CC2+1+(((z14)^(-1))*z24);
    DEN3=  CC3+R3*(z14^(-1));
            a1 =1/DEN1;
            a2=((z14^(-1))*(z24^(-1)))/DEN1;
    a=[a1 a2];
            b1 = (CC2+1)/DEN2;
            b2 = (((z14)^(-1))*z24)/DEN2;
    b=[b1 b2];
            c1 = CC3/DEN3;
            c2= (R3*(z14^(-1)))/DEN3;
    c=[c1 c2];
    d=0;
    
elseif ((i==1)&&(j==2))
    DEN1 = 1+((z14^(-1))*(z24^(-1))); 
    DEN2 = CC2+1+(z23^(-1))+((z14)^(-1))*(z23+z24*(z23^(-2)));
    DEN3 =  (CC3/R3)+(z14^(-1))*(1+z23^(-1));
   % DEN6 = z23+Lmax;
            a1 =1/DEN1;
            a2=((z14^(-1))*(z24^(-1)))/DEN1;
    a=[a1 a2];
            b1 = (CC2+1)/DEN2;
            b2 = (z23^(-1))/DEN2;
            b3 = (((z14)^(-1))*z23)/DEN2;
            b4 = ((z14)^(-1))*(z24*(z23^(-2)))/DEN2;
    b=[b1 b2 b3 b4];
            c1 = CC3/(R3*DEN3);
            c2 = (z14^(-1))/DEN3;
            c3 = (z14^(-1))*(z23^(-1))/DEN3;
    c=[c1 c2 c3];
             
    d=0; 
elseif ((i==1)&&(j==3))  
    
    DEN1 = 1+((z14^(-1))*(z24^(-1))); 
    DEN2 = CC2+1+(z22^(-1))+(z23^(-1))+((z14)^(-1))*(z22+z23*z22^(-2)+z24*(z23^(-2)));%(z22^(-1))+(z23^(-1))+
    DEN3 =  (CC3/R3)+(z14^(-1))*(1+z22^(-1)+z23^(-1));
    
            a1 =1/DEN1;
            a2=((z14^(-1))*(z24^(-1)))/DEN1;
    a=[a1 a2];
            b1 = (CC2+1)/DEN2;
            b2 = (z22^(-1))/DEN2;
            b3 = (z23^(-1))/DEN2;
            b4 = (((z14)^(-1))*z22)/DEN2;
            b5 = ((z14)^(-1))*(z23*(z22^(-2)))/DEN2;
            b6 = ((z14)^(-1))*(z24*(z23^(-2)))/DEN2;
                       
    b=[b1 b2 b3 b4 b5 b6];
            c1 = CC3/(R3*DEN3);
            c2 = (z14^(-1))/DEN3;
            c3 = (z14^(-1))*(z22^(-1))/DEN3;
            c4 = (z14^(-1))*(z23^(-1))/DEN3;
    c=[c1 c2 c3 c4];
         
    d=0; 
    elseif ((i==1)&&(j==4))  
    
    DEN1 = 1+((z14^(-1))*(z24^(-1))); 
    DEN2 = CC2+1+((z21)^(-1))+((z22)^(-1))+((z23)^(-1))+((z14)^(-1))*(z21+z22*((z21)^(-2))+z23*((z22)^(-2))+z24*((z23)^(-2)));
    DEN3 =  (CC3/R3)+(z14^(-1))*(1+z21^(-1)+z22^(-1)+z23^(-1));
    
            a1 =1/DEN1;
            a2=((z14^(-1))*(z24^(-1)))/DEN1;
    a=[a1 a2];
            b1 = (CC2+1)/DEN2;
            b2 = ((z21)^(-1))/DEN2;
            b3 = ((z22)^(-1))/DEN2;
            b4 = ((z23)^(-1))/DEN2;
            b5 = (((z14)^(-1))*z21)/DEN2;
            b6 = (((z14)^(-1))*z22*((z21)^(-2)))/DEN2;
            b7 = (((z14)^(-1))*z23*((z22)^(-2)))/DEN2;
            b8 = (((z14)^(-1))*z24*((z23)^(-2)))/DEN2;
            
    b=[b1 b2 b3 b4 b5 b6 b7 b8];
            c1 = CC3/(R3*DEN3);
            c2 = (z14^(-1))/DEN3;
            c3 = (z14^(-1))*(z21^(-1))/DEN3;
            c4 = (z14^(-1))*(z22^(-1))/DEN3;
            c5 = (z14^(-1))*(z23^(-1))/DEN3;
    c=[c1 c2 c3 c4 c5];
       d=0; 
elseif ((i==2)&&(j==1))
    DEN1 = 1+((z14^(-1))*(z24^(-1))); 
    DEN2 = CC2+1+(((z14)^(-1))*z24);
    DEN3=  CC3+R3*(z14^(-1));
    DEN4 = CC1+1+(z13^(-1));
    
            a1 =1/DEN1;
            a2=((z14^(-1))*(z24^(-1)))/DEN1;
    a=[a1 a2];
            b1 = (CC2+1)/DEN2;
            b2 = (((z14)^(-1))*z24)/DEN2;
    b=[b1 b2];
            c1 = CC3/DEN3;
            c2 = (R3*(z14^(-1)))/DEN3;
    c=[c1 c2];
            d1 = (CC1+1)/DEN4;
            d2 = (z13^(-1))/DEN4;
     d=[d1 d2];
elseif ((i==2)&&(j==2))
    DEN1 = 1+((z14^(-1))*(z24^(-1))); 
    DEN2 = CC2+1+(z23^(-1))+((z14)^(-1))*(z23+z24*(z23^(-2)));
    DEN3=  (CC3/R3)+(z14^(-1))*(1+z23^(-1));
    DEN4 = CC1+1+(z13^(-1));
    
            a1 =1/DEN1;
            a2=((z14^(-1))*(z24^(-1)))/DEN1;
    a=[a1 a2];
            b1 = (CC2+1)/DEN2;
            b2 = (z23^(-1))/DEN2;
            b3 = (((z14)^(-1))*z23)/DEN2;
            b4 = ((z14)^(-1))*(z24*(z23^(-2)))/DEN2;
    b=[b1 b2 b3 b4];
            c1 = CC3/(R3*DEN3);
            c2 = ((z14^(-1)))/DEN3;
            c3 = ((z14^(-1))*((z23)^(-1)))/DEN3;
    c=[c1 c2 c3];
            d1 = (CC1+1)/DEN4;
            d2 = (z13^(-1))/DEN4;
     d=[d1 d2];

elseif ((i==2)&&(j==3))
    DEN1 = 1+((z14^(-1))*(z24^(-1))); 
    DEN2 = CC2+1+(z22^(-1))+(z23^(-1))+((z14)^(-1))*(z22+z23*z22^(-2)+z24*(z23^(-2)));
    DEN3=  (CC3/R3)+(z14^(-1))*(1+(z22^(-1))+(z23^(-1)));
    DEN4 = (CC1+1+z13^(-1));
    
            a1 =1/DEN1;
            a2=((z14^(-1))*(z24^(-1)))/DEN1;
    a=[a1 a2];
            b1 = (CC2+1)/DEN2;
            b2 = (z22^(-1))/DEN2;
            b3 = (z23^(-1))/DEN2;
            b4 = (((z14)^(-1))*z22)/DEN2;
            b5 = ((z14)^(-1))*(z23*(z22^(-2)))/DEN2;
            b6 = ((z14)^(-1))*(z24*(z23^(-2)))/DEN2;
                       
    b=[b1 b2 b3 b4 b5 b6];
            c1 = CC3/(R3*DEN3);
            c2 = ((z14^(-1)))/DEN3;
            c3 = ((z14^(-1))*((z22)^(-1)))/DEN3;
            c4 = ((z14^(-1))*((z23)^(-1)))/DEN3;
    c=[c1 c2 c3 c4];
            d1 = (CC1+1)/DEN4;
            d2 = (z13^(-1))/DEN4;
     d=[d1 d2];
            
elseif ((i==2)&&(j==4))  %%%%%%%%%%%%%%%%%%%%%%%%%%%% CASE VIII %%%% CASE VIII
    DEN1 = 1+((z14^(-1))*(z24^(-1))); 
    DEN2 = CC2+1+((z21)^(-1))+((z22)^(-1))+((z23)^(-1))+((z14)^(-1))*(z21+z22*((z21)^(-2))+z23*((z22)^(-2))+z24*((z23)^(-2)));
    DEN3=  (CC3/R3)+(z14^(-1))*(1+(z21^(-1))+(z22^(-1))+(z23^(-1)));
    DEN4 = (CC1+1+z13^(-1));
    
            a1 =1/DEN1;
            a2=((z14^(-1))*(z24^(-1)))/DEN1;
    a=[a1 a2];
            b1 = (CC2+1)/DEN2;
            b2 = ((z21)^(-1))/DEN2;
            b3 = ((z22)^(-1))/DEN2;
            b4 = ((z23)^(-1))/DEN2;
            b5 = (((z14)^(-1))*z21)/DEN2;
            b6 = (((z14)^(-1))*z22*((z21)^(-2)))/DEN2;
            b7 = (((z14)^(-1))*z23*((z22)^(-2)))/DEN2;
            b8 = (((z14)^(-1))*z24*((z23)^(-2)))/DEN2;
            
    b=[b1 b2 b3 b4 b5 b6 b7 b8];
            c1 = CC3/(R3*DEN3);
            c2 = ((z14^(-1)))/DEN3;
            c3 = ((z14^(-1))*((z21)^(-1)))/DEN3;
            c4 = ((z14^(-1))*((z22)^(-1)))/DEN3;
            c5 = ((z14^(-1))*((z23)^(-1)))/DEN3;
    c=[c1 c2 c3 c4 c5];
             d1 = (CC1+1)/DEN4;
             d2 = (z13^(-1))/DEN4;
     d=[d1 d2];
    
elseif ((i==3)&&(j==1)) %%%%%%%%%%%%%%%%%%%%%%%%%%%% CASE IX %%%% CASE IX
    DEN1 = 1+((z14^(-1))*(z24^(-1))); 
    DEN2 = CC2+1+((z14)^(-1))*z24;
    DEN3=  CC3+R3*(z14^(-1));
    DEN4 = CC1+1+z12^(-1)+z13^(-1);
    
            a1 =1/DEN1;
            a2=((z14^(-1))*(z24^(-1)))/DEN1;
    a=[a1 a2];
            b1 = (CC2+1)/DEN2;
            b2 = (((z14)^(-1))*z24)/DEN2;
    b=[b1 b2];
            c1 = CC3/DEN3;
            c2 = (R3*(z14^(-1)))/DEN3;
    c=[c1 c2];
            d1 = (CC1+1)/DEN4;
            d2 = (z12^(-1))/DEN4;
            d3 = (z13^(-1))/DEN4;
    d=[d1 d2 d3];
            
elseif ((i==3)&&(j==2)) %%%%%%%%%%%%%%%%%%%%%%%%%%%% CASE X %%%% CASE X
    DEN1 = 1+((z14^(-1))*(z24^(-1))); 
    DEN2 = CC2+1+(z23^(-1))+((z14)^(-1))*(z23+z24*(z23^(-2)));
    DEN3=  (CC3/R3)+(z14^(-1))*(1+z23^(-1));
    DEN4 = CC1+1+z12^(-1)+z13^(-1);

            a1 =1/DEN1;
            a2=((z14^(-1))*(z24^(-1)))/DEN1;
    a=[a1 a2];
            b1 = (CC2+1)/DEN2;
            b2 = (z23^(-1))/DEN2;
            b3 = (((z14)^(-1))*z23)/DEN2;
            b4 = ((z14)^(-1))*(z24*(z23^(-2)))/DEN2;
    b=[b1 b2 b3 b4];
            c1 = CC3/(R3*DEN3);
            c2 = (z14^(-1))/DEN3;
            c3 = ((z14^(-1))*(z23^(-1)))/DEN3;
    c=[c1 c2 c3];
            d1 = (CC1+1)/DEN4;
            d2 = (z12^(-1))/DEN4;
            d3 = (z13^(-1))/DEN4;
    d=[d1 d2 d3];
 
elseif ((i==3)&&(j==3)) %%%%%%%%%%%%%%%%%%%%%%%%%%%% CASE XI %%%% CASE XI
    DEN1 = 1+((z14^(-1))*(z24^(-1))); 
    DEN2 = CC2+1+(z22^(-1))+(z23^(-1))+((z14)^(-1))*(z22+z23*z22^(-2)+z24*(z23^(-2)));
    DEN3=  (CC3/R3)+(z14^(-1))*(1+(z22^(-1))+(z23^(-1)));
    DEN4 = CC1+1+z12^(-1)+z13^(-1);
            a1 =1/DEN1;
            a2=((z14^(-1))*(z24^(-1)))/DEN1;
    a=[a1 a2];
            b1 = (CC2+1)/DEN2;
            b2 = (z22^(-1))/DEN2;
            b3 = (z23^(-1))/DEN2;
            b4 = (((z14)^(-1))*z22)/DEN2;
            b5 = ((z14)^(-1))*(z23*(z22^(-2)))/DEN2;
            b6 = ((z14)^(-1))*(z24*(z23^(-2)))/DEN2;
                       
    b=[b1 b2 b3 b4 b5 b6];
            c1 = CC3/(R3*DEN3);
            c2 = (z14^(-1))/DEN3;
            c3 = ((z14^(-1))*(z22^(-1)))/DEN3;
            c4 = ((z14^(-1))*(z23^(-1)))/DEN3;
    c=[c1 c2 c3 c4];
            d1 = (CC1+1)/DEN4;
            d2 = (z12^(-1))/DEN4;
            d3 = (z13^(-1))/DEN4;
    d=[d1 d2 d3];
            
    elseif ((i==3)&&(j==4)) %%%%%%%%%%%%%%%%%%%%%%%%%%%% CASE XII %%%% CASE XII
    DEN1 = 1+((z14^(-1))*(z24^(-1))); 
    DEN2 = CC2+1+((z21)^(-1))+((z22)^(-1))+((z23)^(-1))+((z14)^(-1))*(z21+z22*((z21)^(-2))+z23*((z22)^(-2))+z24*((z23)^(-2)));
    DEN3=  (CC3/R3)+(z14^(-1))*(1+(z21^(-1))+(z22^(-1))+(z23^(-1)));
    DEN4 = CC1+1+z12^(-1)+z13^(-1);
            a1 =1/DEN1;
            a2=((z14^(-1))*(z24^(-1)))/DEN1;
    a=[a1 a2];
            b1 = (CC2+1)/DEN2;
            b2 = ((z21)^(-1))/DEN2;
            b3 = ((z22)^(-1))/DEN2;
            b4 = ((z23)^(-1))/DEN2;
            b5 = (((z14)^(-1))*z21)/DEN2;
            b6 = (((z14)^(-1))*z22*((z21)^(-2)))/DEN2;
            b7 = (((z14)^(-1))*z23*((z22)^(-2)))/DEN2;
            b8 = (((z14)^(-1))*z24*((z23)^(-2)))/DEN2;
            
    b=[b1 b2 b3 b4 b5 b6 b7 b8];
    
            c1 = CC3/(R3*DEN3);
            c2 = (z14^(-1))/DEN3;
            c3 = ((z14^(-1))*(z21^(-1)))/DEN3;
            c4 = ((z14^(-1))*(z22^(-1)))/DEN3;
            c5 = ((z14^(-1))*(z23^(-1)))/DEN3;
    c=[c1 c2 c3 c4 c5];
            d1 = (CC1+1)/DEN4;
            d2 = (z12^(-1))/DEN4;
            d3 = (z13^(-1))/DEN4;
    d=[d1 d2 d3];
            
elseif ((i==4)&&(j==1)) %%%%%%%%%%%%%%%%%%%%%%%%%%%% CASE XIII %%%% CASE XIII
    DEN1 = 1+((z14^(-1))*(z24^(-1))); 
    DEN2 = CC2+1+((z14)^(-1))*z24;
    DEN3=  CC3+R3*(z14^(-1));
    DEN4 = CC1+1+z11^(-1)+z12^(-1)+z13^(-1);
            a1 =1/DEN1;
            a2=((z14^(-1))*(z24^(-1)))/DEN1;
    a=[a1 a2];
            b1 = (CC2+1)/DEN2;
            b2 = (((z14)^(-1))*z24)/DEN2;
    b=[b1 b2];
            c1 = CC3/DEN3;
            c2 = (R3*(z14^(-1)))/DEN3;
    c=[c1 c2];
            d1 = (CC1+1)/DEN4;
            d2 = (z11^(-1))/DEN4;
            d3 = (z12^(-1))/DEN4;
            d4 = (z13^(-1))/DEN4;
    d=[d1 d2 d3 d4];
           
     elseif ((i==4)&&(j==2)) %%%%%%%%%%%%%%%%%%%%%%%%%%%% CASE XIV %%%% CASE XIV
    DEN1 = 1+((z14^(-1))*(z24^(-1))); 
    DEN2 = CC2+1+(z23^(-1))+((z14)^(-1))*(z23+z24*(z23^(-2)));
    DEN3=  (CC3/R3)+(z14^(-1))*(1+z23^(-1));
    DEN4 = CC1+1+z11^(-1)+z12^(-1)+z13^(-1);
            a1 =1/DEN1;
            a2=((z14^(-1))*(z24^(-1)))/DEN1;
    a=[a1 a2];
            b1 = (CC2+1)/DEN2;
            b2 = (z23^(-1))/DEN2;
            b3 = (((z14)^(-1))*z23)/DEN2;
            b4 = ((z14)^(-1))*(z24*(z23^(-2)))/DEN2;
    b=[b1 b2 b3 b4];
            c1 = CC3/(R3*DEN3);
            c2 = (z14^(-1))/DEN3;
            c3 = ((z14^(-1))*(z23^(-1)))/DEN3;
    c=[c1 c2 c3];
            d1 = (CC1+1)/DEN4;
            d2 = (z11^(-1))/DEN4;
            d3 = (z12^(-1))/DEN4;
            d4 = (z13^(-1))/DEN4;
    d=[d1 d2 d3 d4];
               
    elseif ((i==4)&&(j==3)) %%%%%%%%%%%%%%%%%%%%%%%%%%%% CASE XV %%%% CASE XV
    DEN1 = 1+((z14^(-1))*(z24^(-1))); 
    DEN2 = CC2+1+(z22^(-1))+(z23^(-1))+((z14)^(-1))*(z22+z23*(z22^(-2))+z24*(z23^(-2)));
    DEN3=  (CC3/R3)+(z14^(-1))*(1+(z22^(-1))+(z23^(-1)));
    DEN4 = CC1+1+(z11^(-1))+(z12^(-1))+(z13^(-1));
            a1 =1/DEN1;
            a2=((z14^(-1))*(z24^(-1)))/DEN1;
    a=[a1 a2];
            b1 = (CC2+1)/DEN2;
            b2 = (z22^(-1))/DEN2;
            b3 = (z23^(-1))/DEN2;
            b4 = (((z14)^(-1))*z22)/DEN2;
            b5 = ((z14)^(-1))*(z23*(z22^(-2)))/DEN2;
            b6 = ((z14)^(-1))*(z24*(z23^(-2)))/DEN2;
                       
    b=[b1 b2 b3 b4 b5 b6];
            c1 = CC3/(R3*DEN3);
            c2 = (z14^(-1))/DEN3;
            c3 = ((z14^(-1))*(z22^(-1)))/DEN3;
            c4 = ((z14^(-1))*(z23^(-1)))/DEN3;
    c=[c1 c2 c3 c4];
            d1 = (CC1+1)/DEN4;
            d2 = (z11^(-1))/DEN4;
            d3 = (z12^(-1))/DEN4;
            d4 = (z13^(-1))/DEN4;
    d=[d1 d2 d3 d4];
          
    elseif ((i==4)&&(j==4)) %%%%%%%%%%%%%%%%%%%%%%%%%%%% CASE XVI %%%% CASE XVI
    DEN1 = 1+((z14^(-1))*(z24^(-1))); 
    DEN2 = CC2+1+((z21)^(-1))+((z22)^(-1))+((z23)^(-1))+((z14)^(-1))*(z21+z22*((z21)^(-2))+z23*((z22)^(-2))+z24*((z23)^(-2)));
    DEN3=  (CC3/R3)+(z14^(-1))*(1+(z21^(-1))+(z22^(-1))+(z23^(-1)));
    DEN4 = CC1+1+(z11^(-1))+(z12^(-1))+(z13^(-1));
            a1 =1/DEN1;
            a2=((z14^(-1))*(z24^(-1)))/DEN1;
    a=[a1 a2];
            b1 = (CC2+1)/DEN2;
            b2 = ((z21)^(-1))/DEN2;
            b3 = ((z22)^(-1))/DEN2;
            b4 = ((z23)^(-1))/DEN2;
            b5 = (((z14)^(-1))*z21)/DEN2;
            b6 = (((z14)^(-1))*z22*((z21)^(-2)))/DEN2;
            b7 = (((z14)^(-1))*z23*((z22)^(-2)))/DEN2;
            b8 = (((z14)^(-1))*z24*((z23)^(-2)))/DEN2;
            
    b=[b1 b2 b3 b4 b5 b6 b7 b8];
            c1 = CC3/(R3*DEN3);
            c2 = (z14^(-1))/DEN3;
            c3 = ((z14^(-1))*(z21^(-1)))/DEN3;
            c4 = ((z14^(-1))*(z22^(-1)))/DEN3;
            c5 = ((z14^(-1))*(z23^(-1)))/DEN3;
    c=[c1 c2 c3 c4 c5];
            d1 = (CC1+1)/DEN4;
            d2 = (z11^(-1))/DEN4;
            d3 = (z12^(-1))/DEN4;
            d4 = (z13^(-1))/DEN4;
    d=[d1 d2 d3 d4];
           
    
end


end

